@@ -25,12 +25,11 @@ namespace PdfMod
 
         private static string [] require_doc_actions = new string[] {
             "SaveAction", "SaveAsAction", "PropertiesAction", "UndoAction", "RedoAction", "ZoomFitAction",
-            "SelectAllAction", "SelectEvensAction", "SelectOddsAction", "SelectMatchingAction"
+            "SelectAllAction", "SelectEvensAction", "SelectOddsAction", "SelectMatchingAction", "InsertFromAction", "ExportImagesAction"
         };
 
         private static string [] require_page_actions = new string[] {
             "RemoveAction", "ExtractAction", "RotateRightAction", "RotateLeftAction"
-            //, "ExportImagesAction"
         };
 
         public UndoManager UndoManager {
@@ -44,6 +43,7 @@ namespace PdfMod
 
             AddImportant (
                 new ActionEntry ("OpenAction",   Gtk.Stock.Open,   null, "<control>O", Catalog.GetString ("Open a document"), OnOpen),
+                new ActionEntry ("InsertFromAction", Gtk.Stock.Add, Catalog.GetString("_Insert From..."), null, Catalog.GetString("Insert pages from another document"), OnInsertFrom),
                 new ActionEntry ("SaveAction",   Gtk.Stock.Save,   null, "<control>S", Catalog.GetString ("Save changes to this document, overwriting the existing file"), OnSave),
                 new ActionEntry ("SaveAsAction", Gtk.Stock.SaveAs, null, "<control><shift>S", Catalog.GetString ("Save this document to a new file"), OnSaveAs),
 
@@ -54,7 +54,7 @@ namespace PdfMod
                 new ActionEntry ("ExtractAction", Gtk.Stock.New, null, null, null, OnExtractPages),
                 new ActionEntry ("RotateRightAction", null, Catalog.GetString ("Rotate Right"), "bracketright", Catalog.GetString ("Rotate right"), OnRotateRight),
                 new ActionEntry ("RotateLeftAction", null, Catalog.GetString ("Rotate Left"), "bracketleft", Catalog.GetString ("Rotate left"), OnRotateLeft),
-                new ActionEntry ("ExportImagesAction", null, Catalog.GetString ("Export Images..."), null, null, OnExportImages),
+                new ActionEntry ("ExportImagesAction", null, Catalog.GetString ("Export Images"), null, Catalog.GetString ("Save all images in this document to a new folder"), OnExportImages),
 
                 new ActionEntry ("EditMenuAction", null, Catalog.GetString ("_Edit"), null, null, null),
                 new ActionEntry ("SelectAllAction", Stock.SelectAll, null, "<control>A", null, OnSelectAll),
@@ -81,11 +81,9 @@ namespace PdfMod
                 new ToggleActionEntry ("ViewToolbar", null, Catalog.GetString ("Toolbar"), null, null, OnViewToolbar, true)
             );
 
-            // Not ready/finished yet
-            UpdateAction ("ExportImagesAction", false);
-
             this["RotateRightAction"].IconName = "object-rotate-right";
             this["RotateLeftAction"].IconName = "object-rotate-left";
+            this["ExportImagesAction"].IconName = "image-x-generic";
 
             // Don't show HelpAction unless we can at least access some of the GNOME api
             UpdateAction ("HelpAction", false);
@@ -188,6 +186,25 @@ namespace PdfMod
             }
         }
 
+        private void OnInsertFrom (object o, EventArgs args)
+        {
+            var chooser = new Gtk.FileChooserDialog (Catalog.GetString ("Select PDF"), app.Window, FileChooserAction.Open);
+            chooser.AddFilter (GtkUtilities.GetFileFilter ("PDF Documents", new string [] {"pdf"}));
+            chooser.AddFilter (GtkUtilities.GetFileFilter (Catalog.GetString ("All Files"), new string [] {"*"}));
+            chooser.SelectMultiple = false;
+            chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
+            chooser.AddButton (Stock.Open, ResponseType.Ok);
+            chooser.DefaultResponse = ResponseType.Ok;
+
+            var response = chooser.Run ();
+            string filename = chooser.Filename;
+            chooser.Destroy();
+
+            if (response == (int)ResponseType.Ok) {
+                app.Document.AddFromUri (new Uri (filename));
+            }
+        }
+
         private void OnSave (object o, EventArgs args)
         {
             app.Document.Save (app.Document.SuggestedSavePath);
@@ -280,8 +297,7 @@ namespace PdfMod
 
         private void OnExportImages (object o, EventArgs args)
         {
-            var pages = app.IconView.SelectedPages.ToList ();
-            var action = new ExportImagesAction (app.Document, pages);
+            var action = new ExportImagesAction (app.Document, app.Document.Pages);
             if (action.ExportableImageCount == 0) {
                 Log.Information ("Found zero exportable images in the selected pages");
                 return;
@@ -290,16 +306,21 @@ namespace PdfMod
             var export_path_base = Path.Combine (
                 Path.GetDirectoryName (app.Document.SuggestedSavePath),
                 // Translators: This is used for creating a folder name, be careful!
-                String.Format (Catalog.GetString ("{0} - Images for {1}"), app.Document.Title ?? app.Document.Filename, GetPageSummary (pages, 10))
+                //String.Format (Catalog.GetString ("{0} - Images for {1}"), app.Document.Title ?? app.Document.Filename, GetPageSummary (pages, 10))
+                app.Document.Title ?? System.IO.Path.GetFileNameWithoutExtension (app.Document.Filename)
             );
 
             var export_path = export_path_base;
             int i = 1;
-            while (Directory.Exists (export_path)) {
+            while (Directory.Exists (export_path) && i < 100) {
                 export_path = String.Format ("{0} ({1})", export_path_base, i++);
             }
 
-            Directory.CreateDirectory (export_path);
+            try {
+                Directory.CreateDirectory (export_path);
+            } catch (Exception e) {
+                Hyena.Log.Exception (e);
+            }
 
             action.Do (export_path);
             System.Diagnostics.Process.Start (export_path);
@@ -337,7 +358,7 @@ namespace PdfMod
                 string help_dir = null;
                 foreach (var dir in new string [] { "/usr/share/gnome/help/", "/usr/local/share/gnome/help/", "docs/" }) {
                     help_dir = dir;
-                    if (System.IO.Directory.Exists (dir)) {
+                    if (System.IO.Directory.Exists (dir + "pdfmod/")) {
                         break;
                     }
                 }
@@ -377,14 +398,28 @@ namespace PdfMod
 
             var dialog = new Gtk.AboutDialog () {
                 ProgramName = "PDF Mod",
-                Version = "0.3",
+                Version = "0.4",
                 Website = WIKI_URL,
                 WebsiteLabel = Catalog.GetString ("Visit Website"),
-                Authors = new string [] { "Gabriel Burt", "", "Contributions from:", "Sandy Armstrong", "Aaron Bockover", "Olivier Le Thanh Duong" },
+                Authors = new string [] {
+                    "Gabriel Burt", "",
+                    "Contributions from:",
+                    " • Sandy Armstrong",
+                    " • Aaron Bockover",
+                    " • Olivier Le Thanh Duong",
+                    " • Julien Rebetez",
+                    " • Igor Vatavuk"
+                },
+                Documenters = new string [] { "Gabriel Burt" },
+                Artists = new string [] { "Kalle Persson" },
                 Copyright = "Copyright 2009 Novell Inc.",
                 TranslatorCredits = Catalog.GetString ("translator-credits")
             };
 
+            try {
+                dialog.Logo = Gtk.IconTheme.Default.LoadIcon ("pdfmod", 256, 0);
+            } catch {}
+
             string [] license_paths = new string [] {
                 "/usr/share/doc/packages/pdfmod/COPYING",
                 "/usr/local/share/doc/packages/pdfmod/COPYING",
