@@ -23,6 +23,7 @@ namespace PdfMod
             ThreadAssist.ProxyToMainHandler = RunIdle;
 
             Hyena.Log.Debugging = true;
+            Hyena.Log.Notify += OnLogNotify;
             Hyena.Log.DebugFormat ("Starting PdfMod");
 
             Gtk.Application.Init ();
@@ -61,6 +62,7 @@ namespace PdfMod
 
             Window = new Gtk.Window (WindowType.Toplevel);
             Window.Title = Catalog.GetString ("PDF Mod");
+            Window.DefaultIconName = "pdfmod";
             Window.SetSizeRequest (640, 480);
             Window.DeleteEvent += delegate (object o, DeleteEventArgs args) {
                 Quit ();
@@ -69,8 +71,8 @@ namespace PdfMod
 
             // PDF Icon View
             IconView = new PdfIconView (this);
-            var IconView_sw = new Gtk.ScrolledWindow ();
-            IconView_sw.Child = IconView;
+            var iconview_sw = new Gtk.ScrolledWindow ();
+            iconview_sw.Child = IconView;
 
             query_box = new QueryBox (this) { NoShowAll = true };
             query_box.Hide ();
@@ -78,8 +80,9 @@ namespace PdfMod
             // Status bar
             StatusBar = new Gtk.Statusbar () { HasResizeGrip = true };
             size_label = new Label ();
-            size_label.Xalign = 0;
-            StatusBar.PackStart (size_label, false, false, 0);
+            size_label.Xalign = 0.0f;
+            StatusBar.PackStart (size_label, true, true, 6);
+            StatusBar.ReorderChild (size_label, 0);
 
             // ActionManager
             ActionManager = new Hyena.Gui.ActionManager ();
@@ -103,7 +106,7 @@ namespace PdfMod
             vbox.PackStart (HeaderToolbar, false, false, 0);
             vbox.PackStart (EditorBox, false, false, 0);
             vbox.PackStart (query_box, false, false, 0);
-            vbox.PackStart (IconView_sw, true, true, 0);
+            vbox.PackStart (iconview_sw, true, true, 0);
             vbox.PackStart (StatusBar, false, true, 0);
             Window.Add (vbox);
 
@@ -254,25 +257,59 @@ namespace PdfMod
         private void UpdateForDocument ()
         {
             var current_size = Document.FileSize;
+            string size_str = null;
             if (original_size_string == null) {
-                size_label.Text = original_size_string = new Hyena.Query.FileSizeQueryValue (current_size).ToUserQuery ();
+                size_str = original_size_string = new Hyena.Query.FileSizeQueryValue (current_size).ToUserQuery ();
                 original_size = current_size;
             } else if (current_size == original_size) {
-                size_label.Text = original_size_string;
+                size_str = original_size_string;
             } else {
                 string current_size_string = new Hyena.Query.FileSizeQueryValue (current_size).ToUserQuery ();
                 if (current_size_string == original_size_string) {
-                    size_label.Text = original_size_string;
+                    size_str = original_size_string;
                 } else {
-                    size_label.Text = String.Format (Catalog.GetString ("{0} (originally {1})"), current_size_string, original_size_string);
+                    // Translators: this string is used to show current/original file size, eg "2 MB (originally 1 MB)"
+                    size_str = String.Format (Catalog.GetString ("{0} (originally {1})"), current_size_string, original_size_string);
                 }
             }
 
+            size_label.Text = String.Format ("{0} \u2013 {1}",
+                String.Format (Catalog.GetPluralString ("{0} page", "{0} pages", Document.Count), Document.Count),
+                size_str
+            );
+
             var title = Document.Title;
             var filename = Document.Filename;
             Window.Title = title == null ? filename : String.Format ("{0} ({1})", title, filename);
         }
 
+        private static void OnLogNotify (LogNotifyArgs args)
+        {
+            Gtk.Window window = null;
+            Gtk.MessageType mtype;
+            var entry = args.Entry;
+
+            switch (entry.Type) {
+                case LogEntryType.Warning:
+                    mtype = Gtk.MessageType.Warning;
+                    break;
+                case LogEntryType.Information:
+                    mtype = Gtk.MessageType.Info;
+                    break;
+                case LogEntryType.Error:
+                default:
+                    mtype = Gtk.MessageType.Error;
+                    break;
+            }
+
+            Hyena.Widgets.HigMessageDialog dialog = new Hyena.Widgets.HigMessageDialog (
+                null, Gtk.DialogFlags.Modal, mtype, Gtk.ButtonsType.Close, entry.Message, entry.Details);
+
+            dialog.Title = String.Empty;
+            dialog.Run ();
+            dialog.Destroy ();
+        }
+
         private static void InitCatalog (params string [] dirs)
         {
             foreach (var dir in dirs) {
